@@ -1,27 +1,19 @@
-/* coding.cpp                                
- *
- * Copyright (C) 2003 Sawtooth Consulting Ltd.
- *
- * This file is part of yaSSL.
- *
- * yaSSL is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * There are special exceptions to the terms and conditions of the GPL as it
- * is applied to yaSSL. View the full text of the exception in the file
- * FLOSS-EXCEPTIONS in the directory of this software distribution.
- *
- * yaSSL is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- */
+/*
+   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 /* coding.cpp implements hex and base64 encoding/decoing
 */
@@ -114,11 +106,12 @@ void HexDecoder::Decode()
         // sanity checks
         assert( b  < sizeof(hexDecode)/sizeof(hexDecode[0]) );
         assert( b2 < sizeof(hexDecode)/sizeof(hexDecode[0]) );
-        assert( b != bad && b2 != bad );
 
         b  = hexDecode[b];
         b2 = hexDecode[b2];
         
+        assert( b != bad && b2 != bad );
+        
         decoded_[i++] = (b << 4) | b2;
         bytes -= 2;
     }
@@ -131,8 +124,7 @@ void HexDecoder::Decode()
 void Base64Encoder::Encode()
 {
     word32 bytes = plain_.size();
-    word32 outSz = bytes * 4 / 3;
-    outSz += (outSz % 4);           // 4 byte integrals         
+    word32 outSz = (bytes + 3 - 1) / 3 * 4;
 
     outSz += (outSz + pemLineSz - 1) / pemLineSz;  // new lines
     encoded_.New(outSz);
@@ -159,7 +151,7 @@ void Base64Encoder::Encode()
 
         bytes -= 3;
 
-        if ((++j % 16) == 0)
+        if ((++j % 16) == 0 && bytes)
             encoded_[i++] = '\n';
     }
 
@@ -192,7 +184,7 @@ void Base64Decoder::Decode()
 {
     word32 bytes = coded_.size();
     word32 plainSz = bytes - ((bytes + (pemLineSz - 1)) / pemLineSz); 
-    plainSz = (plainSz * 3 + 3) / 4;
+    plainSz = ((plainSz * 3) / 4) + 3;
     decoded_.New(plainSz);
 
     word32 i = 0;
@@ -236,11 +228,18 @@ void Base64Decoder::Decode()
         if ((++j % 16) == 0) {
             byte endLine = coded_.next();
             bytes--;
+            while (endLine == ' ') {        // remove possible whitespace
+                endLine = coded_.next();
+                bytes--;
+            }
             if (endLine == '\r') {
                 endLine = coded_.next();
                 bytes--;
             }
-            assert(endLine == '\n');
+            if (endLine != '\n') {
+                coded_.SetError(PEM_E); 
+                return;
+            }
         }
     }
 
