@@ -28,16 +28,7 @@ extern "C"
   extern unsigned long max_allowed_packet, net_buffer_length;
 }
 
-#if defined (__WIN__)
-#include "../sql/mysqld.cpp"
-#else
 #include "../sql/mysqld.cc"
-#endif
-
-int check_user(THD *thd, enum enum_server_command command, 
-	       const char *passwd, uint passwd_len, const char *db,
-	       bool check_count);
-void thd_init_client_charset(THD *thd, uint cs_number);
 
 C_MODE_START
 
@@ -47,9 +38,37 @@ C_MODE_START
 #include <sql_common.h>
 #include "embedded_priv.h"
 
+extern unsigned int mysql_server_last_errno;
+extern char mysql_server_last_error[MYSQL_ERRMSG_SIZE];
 static my_bool emb_read_query_result(MYSQL *mysql);
 
 
+extern "C" void unireg_clear(int exit_code)
+{
+  DBUG_ENTER("unireg_clear");
+  clean_up(!opt_help && (exit_code || !opt_bootstrap)); /* purecov: inspected */
+  my_end(opt_endinfo ? MY_CHECK_ERROR | MY_GIVE_INFO : 0);
+  DBUG_VOID_RETURN;
+}
+
+/*
+  Wrapper error handler for embedded server to call client/server error 
+  handler based on whether thread is in client/server context
+*/
+
+static int embedded_error_handler(uint error, const char *str, myf MyFlags)
+{
+  DBUG_ENTER("embedded_error_handler");
+
+  /* 
+    If current_thd is NULL, it means restore_global has been called and 
+    thread is in client context, then call client error handler else call 
+    server error handler.
+  */
+  DBUG_RETURN(current_thd ? my_message_sql(error, str, MyFlags):
+              my_message_no_curses(error, str, MyFlags));
+}
+
 /*
   Reads error information from the MYSQL_DATA and puts
   it into proper MYSQL members
@@ -71,39 +90,54 @@ void embedded_get_error(MYSQL *mysql, MYSQL_DATA *data)
   NET *net= &mysql->net;
   struct embedded_query_result *ei= data->embedded_info;
   net->last_errno= ei->last_errno;
-  strmake(net->last_error, ei->info, sizeof(net->last_error));
+  strmake(net->last_error, ei->info, sizeof(net->last_error)-1);
   memcpy(net->sqlstate, ei->sqlstate, sizeof(net->sqlstate));
   mysql->server_status= ei->server_status;
-  my_free((gptr) data, MYF(0));
+  my_free(data, MYF(0));
 }
 
 static my_bool
 emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
-		     const char *header, ulong header_length,
-		     const char *arg, ulong arg_length, my_bool skip_check,
+		     const uchar *header, ulong header_length,
+		     const uchar *arg, ulong arg_length, my_bool skip_check,
                      MYSQL_STMT *stmt)
 {
   my_bool result= 1;
   THD *thd=(THD *) mysql->thd;
   NET *net= &mysql->net;
+  my_bool stmt_skip= stmt ? stmt->state != MYSQL_STMT_INIT_DONE : FALSE;
+
+  if (!thd)
+  {
+    /* Do "reconnect" if possible */
+    if (mysql_reconnect(mysql) || stmt_skip)
+      return 1;
+    thd= (THD *) mysql->thd;
+  }
+
+#if defined(ENABLED_PROFILING) && defined(COMMUNITY_SERVER)
+  thd->profiling.start_new_query();
+#endif
 
   thd->clear_data_list();
   /* Check that we are calling the client functions in right order */
   if (mysql->status != MYSQL_STATUS_READY)
   {
-    strmov(net->last_error,
-	   ER(net->last_errno=CR_COMMANDS_OUT_OF_SYNC));
-    return 1;
+    set_mysql_error(mysql, CR_COMMANDS_OUT_OF_SYNC, unknown_sqlstate);
+    result= 1;
+    goto end;
   }
 
   /* Clear result variables */
   thd->clear_error();
+  thd->main_da.reset_diagnostics_area();
   mysql->affected_rows= ~(my_ulonglong) 0;
   mysql->field_count= 0;
-  net->last_errno= 0;
+  net_clear_error(net);
   thd->current_stmt= stmt;
 
   thd->store_globals();				// Fix if more than one connect
+  lex_start(thd);
   /* 
      We have to call free_old_query before we start to fill mysql->fields 
      for new query. In the case of embedded server we collect field data
@@ -120,13 +154,20 @@ emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
     arg_length= header_length;
   }
 
-  thd->net.no_send_error= 0;
-  result= dispatch_command(command, thd, (char *) arg, arg_length + 1);
+  result= dispatch_command(command, thd, (char *) arg, arg_length);
   thd->cur_data= 0;
 
   if (!skip_check)
-    result= thd->net.last_errno ? -1 : 0;
+    result= thd->is_error() ? -1 : 0;
 
+  thd->mysys_var= 0;
+
+#if defined(ENABLED_PROFILING) && defined(COMMUNITY_SERVER)
+  thd->profiling.finish_current_query();
+#endif
+
+end:
+  thd->restore_globals();
   return result;
 }
 
@@ -186,7 +227,7 @@ static MYSQL_FIELD *emb_list_fields(MYSQL *mysql)
   res= ((THD*) mysql->thd)->cur_data;
   ((THD*) mysql->thd)->cur_data= 0;
   mysql->field_alloc= res->alloc;
-  my_free((gptr) res,MYF(0));
+  my_free(res,MYF(0));
   mysql->status= MYSQL_STATUS_READY;
   return mysql->fields;
 }
@@ -215,7 +256,7 @@ static my_bool emb_read_prepare_result(MYSQL *mysql, MYSQL_STMT *stmt)
     stmt->fields= mysql->fields;
     stmt->mem_root= res->alloc;
     mysql->fields= NULL;
-    my_free((gptr) res,MYF(0));
+    my_free(res,MYF(0));
   }
 
   return 0;
@@ -252,11 +293,12 @@ static my_bool emb_read_query_result(MYSQL *mysql)
   mysql->warning_count= res->embedded_info->warning_count;
   mysql->server_status= res->embedded_info->server_status;
   mysql->field_count= res->fields;
-  mysql->fields= res->embedded_info->fields_list;
-  mysql->affected_rows= res->embedded_info->affected_rows;
-  mysql->insert_id= res->embedded_info->insert_id;
-  mysql->net.last_errno= 0;
-  mysql->net.last_error[0]= 0;
+  if (!(mysql->fields= res->embedded_info->fields_list))
+  {
+    mysql->affected_rows= res->embedded_info->affected_rows;
+    mysql->insert_id= res->embedded_info->insert_id;
+  }
+  net_clear_error(&mysql->net);
   mysql->info= 0;
 
   if (res->embedded_info->info[0])
@@ -271,7 +313,7 @@ static my_bool emb_read_query_result(MYSQL *mysql)
     thd->cur_data= res;
   }
   else
-    my_free((gptr) res, MYF(0));
+    my_free(res, MYF(0));
 
   return 0;
 }
@@ -279,12 +321,12 @@ static my_bool emb_read_query_result(MYSQL *mysql)
 static int emb_stmt_execute(MYSQL_STMT *stmt)
 {
   DBUG_ENTER("emb_stmt_execute");
-  char header[5];
+  uchar header[5];
   THD *thd;
   my_bool res;
 
   int4store(header, stmt->stmt_id);
-  header[4]= stmt->flags;
+  header[4]= (uchar) stmt->flags;
   thd= (THD*)stmt->mysql->thd;
   thd->client_param_count= stmt->param_count;
   thd->client_params= stmt->params;
@@ -298,7 +340,7 @@ static int emb_stmt_execute(MYSQL_STMT *stmt)
   if (res)
   {
     NET *net= &stmt->mysql->net;
-    set_stmt_errmsg(stmt, net->last_error, net->last_errno, net->sqlstate);
+    set_stmt_errmsg(stmt, net);
     DBUG_RETURN(1);
   }
   DBUG_RETURN(0);
@@ -309,14 +351,12 @@ int emb_read_binary_rows(MYSQL_STMT *stmt)
   MYSQL_DATA *data;
   if (!(data= emb_read_rows(stmt->mysql, 0, 0)))
   {
-    set_stmt_errmsg(stmt, stmt->mysql->net.last_error,
-                    stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate);
+    set_stmt_errmsg(stmt, &stmt->mysql->net);
     return 1;
   }
   stmt->result= *data;
   my_free((char *) data, MYF(0));
-  set_stmt_errmsg(stmt, stmt->mysql->net.last_error,
-                  stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate);
+  set_stmt_errmsg(stmt, &stmt->mysql->net);
   return 0;
 }
 
@@ -330,16 +370,14 @@ int emb_read_rows_from_cursor(MYSQL_STMT *stmt)
   if (res->embedded_info->last_errno)
   {
     embedded_get_error(mysql, res);
-    set_stmt_errmsg(stmt, mysql->net.last_error,
-                    mysql->net.last_errno, mysql->net.sqlstate);
+    set_stmt_errmsg(stmt, &mysql->net);
     return 1;
   }
 
   thd->cur_data= res;
   mysql->warning_count= res->embedded_info->warning_count;
   mysql->server_status= res->embedded_info->server_status;
-  mysql->net.last_errno= 0;
-  mysql->net.last_error[0]= 0;
+  net_clear_error(&mysql->net);
 
   return emb_read_binary_rows(stmt);
 }
@@ -378,14 +416,16 @@ static void emb_free_embedded_thd(MYSQL *mysql)
   thd->clear_data_list();
   thread_count--;
   thd->store_globals();
+  thd->unlink();
   delete thd;
+  my_pthread_setspecific_ptr(THR_THD,  0);
   mysql->thd=0;
 }
 
 static const char * emb_read_statistics(MYSQL *mysql)
 {
   THD *thd= (THD*)mysql->thd;
-  return thd->net.last_error;
+  return thd->is_error() ? thd->main_da.message() : "";
 }
 
 
@@ -476,6 +516,12 @@ int init_embedded_server(int argc, char **argv, char **groups)
 
   my_progname= (char *)"mysql_embedded";
 
+  /*
+    Perform basic logger initialization logger. Should be called after
+    MY_INIT, as it initializes mutexes. Log tables are inited later.
+  */
+  logger.init_base();
+
   if (init_common_variables("my", *argcp, *argvp, (const char **)groups))
   {
     mysql_server_end();
@@ -500,7 +546,10 @@ int init_embedded_server(int argc, char **argv, char **groups)
     return 1;
   }
 
-  error_handler_hook = my_message_sql;
+  /* 
+    set error_handler_hook to embedded_error_handler wrapper.
+  */
+  error_handler_hook= embedded_error_handler;
 
   acl_error= 0;
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
@@ -526,16 +575,12 @@ int init_embedded_server(int argc, char **argv, char **groups)
 
   (void) thr_setconcurrency(concurrency);	// 10 by default
 
-  if (
-#ifdef HAVE_BERKELEY_DB
-      (have_berkeley_db == SHOW_OPTION_YES) ||
-#endif
-      (flush_time && flush_time != ~(ulong) 0L))
-  {
-    pthread_t hThread;
-    if (pthread_create(&hThread,&connection_attrib,handle_manager,0))
-      sql_print_error("Warning: Can't create thread to manage maintenance");
-  }
+  start_handle_manager();
+
+  // FIXME initialize binlog_filter and rpl_filter if not already done
+  //       corresponding delete is in clean_up()
+  if(!binlog_filter) binlog_filter = new Rpl_filter;
+  if(!rpl_filter) rpl_filter = new Rpl_filter;
 
   if (opt_init_file)
   {
@@ -546,6 +591,7 @@ int init_embedded_server(int argc, char **argv, char **groups)
     }
   }
 
+  execute_ddl_log_recovery();
   return 0;
 }
 
@@ -565,10 +611,21 @@ void init_embedded_mysql(MYSQL *mysql, int client_flag)
   init_alloc_root(&mysql->field_alloc, 8192, 0);
 }
 
+/**
+  @brief Initialize a new THD for a connection in the embedded server
+
+  @param client_flag  Client capabilities which this thread supports
+  @return pointer to the created THD object
+
+  @todo
+  This function copies code from several places in the server, including
+  create_new_thread(), and prepare_new_connection_state().  This should
+  be refactored to avoid code duplication.
+*/
 void *create_embedded_thd(int client_flag)
 {
   THD * thd= new THD;
-  thd->thread_id= thread_id++;
+  thd->thread_id= thd->variables.pseudo_thread_id= thread_id++;
 
   thd->thread_stack= (char*) &thd;
   if (thd->store_globals())
@@ -576,12 +633,9 @@ void *create_embedded_thd(int client_flag)
     fprintf(stderr,"store_globals failed.\n");
     goto err;
   }
+  lex_start(thd);
 
-  thd->mysys_var= my_thread_var;
-  thd->dbug_thread_id= my_thread_id();
-  thd->thread_stack= (char*) &thd;
-
-/* TODO - add init_connect command execution */
+  /* TODO - add init_connect command execution */
 
   if (thd->variables.max_join_size == HA_POS_ERROR)
     thd->options |= OPTION_BIG_SELECTS;
@@ -591,7 +645,7 @@ void *create_embedded_thd(int client_flag)
   thd->set_time();
   thd->init_for_queries();
   thd->client_capabilities= client_flag;
-  thd->real_id= (pthread_t)thd->thread_id;
+  thd->real_id= pthread_self();
 
   thd->db= NULL;
   thd->db_length= 0;
@@ -605,6 +659,8 @@ void *create_embedded_thd(int client_flag)
   bzero((char*) &thd->net, sizeof(thd->net));
 
   thread_count++;
+  threads.append(thd);
+  thd->mysys_var= 0;
   return thd;
 err:
   delete(thd);
@@ -624,6 +680,7 @@ int check_embedded_connection(MYSQL *mysql, const char *db)
   strmake(sctx->priv_host, (char*) my_localhost,  MAX_HOSTNAME-1);
   sctx->priv_user= sctx->user= my_strdup(mysql->user, MYF(0));
   result= check_user(thd, COM_CONNECT, NULL, 0, db, true);
+  net_end_statement(thd);
   emb_read_query_result(mysql);
   return result;
 }
@@ -673,8 +730,10 @@ int check_embedded_connection(MYSQL *mysql, const char *db)
 err:
   {
     NET *net= &mysql->net;
-    memcpy(net->last_error, thd->net.last_error, sizeof(net->last_error));
-    memcpy(net->sqlstate, thd->net.sqlstate, sizeof(net->sqlstate));
+    strmake(net->last_error, thd->main_da.message(), sizeof(net->last_error)-1);
+    memcpy(net->sqlstate,
+           mysql_errno_to_sqlstate(thd->main_da.sql_errno()),
+           sizeof(net->sqlstate)-1);
   }
   return result;
 }
@@ -697,9 +756,8 @@ void THD::clear_data_list()
 
 void THD::clear_error()
 {
-  net.last_error[0]= 0;
-  net.last_errno= 0;
-  net.report_error= 0;
+  if (main_da.is_error())
+    main_da.reset_diagnostics_area();
 }
 
 static char *dup_str_aux(MEM_ROOT *root, const char *from, uint length,
@@ -762,23 +820,21 @@ MYSQL_DATA *THD::alloc_new_dataset()
 }
 
 
-/*
-  stores server_status and warning_count in the current
-  query result structures
+/**
+  Stores server_status and warning_count in the current
+  query result structures.
 
-  SYNOPSIS
-  write_eof_packet()
-  thd		current thread
-
-  NOTES
-    should be called to after we get the recordset-result
+  @param thd            current thread
 
+  @note Should be called after we get the recordset-result.
 */
 
-static void write_eof_packet(THD *thd)
+static
+bool
+write_eof_packet(THD *thd, uint server_status, uint total_warn_count)
 {
   if (!thd->mysql)            // bootstrap file handling
-    return;
+    return FALSE;
   /*
     The following test should never be true, but it's better to do it
     because if 'is_fatal_error' is set the server is not going to execute
@@ -786,13 +842,14 @@ static void write_eof_packet(THD *thd)
   */
   if (thd->is_fatal_error)
     thd->server_status&= ~SERVER_MORE_RESULTS_EXISTS;
-  thd->cur_data->embedded_info->server_status= thd->server_status;
+  thd->cur_data->embedded_info->server_status= server_status;
   /*
     Don't send warn count during SP execution, as the warn_list
     is cleared between substatements, and mysqltest gets confused
   */
   thd->cur_data->embedded_info->warning_count=
-    (thd->spcont ? 0 : min(thd->total_warn_count, 65535));
+    (thd->spcont ? 0 : min(total_warn_count, 65535));
+  return FALSE;
 }
 
 
@@ -823,7 +880,7 @@ int Protocol::begin_dataset()
   remove last row of current recordset
 
   SYNOPSIS
-  Protocol_simple::remove_last_row()
+  Protocol_text::remove_last_row()
 
   NOTES
     does the loop from the beginning of the current recordset to
@@ -831,12 +888,12 @@ int Protocol::begin_dataset()
     Not supposed to be frequently called.
 */
 
-void Protocol_simple::remove_last_row()
+void Protocol_text::remove_last_row()
 {
   MYSQL_DATA *data= thd->cur_data;
   MYSQL_ROWS **last_row_hook= &data->data;
-  uint count= data->rows;
-  DBUG_ENTER("Protocol_simple::remove_last_row");
+  my_ulonglong count= data->rows;
+  DBUG_ENTER("Protocol_text::remove_last_row");
   while (--count)
     last_row_hook= &(*last_row_hook)->next;
 
@@ -948,7 +1005,7 @@ bool Protocol::send_fields(List<Item> *list, uint flags)
   }
 
   if (flags & SEND_EOF)
-    write_eof_packet(thd);
+    write_eof_packet(thd, thd->server_status, thd->total_warn_count);
 
   DBUG_RETURN(prepare_for_send(list));
  err:
@@ -965,7 +1022,7 @@ bool Protocol::write()
   return false;
 }
 
-bool Protocol_prep::write()
+bool Protocol_binary::write()
 {
   MYSQL_ROWS *cur;
   MYSQL_DATA *data= thd->cur_data;
@@ -988,52 +1045,96 @@ bool Protocol_prep::write()
   return false;
 }
 
-void
-send_ok(THD *thd,ha_rows affected_rows,ulonglong id,const char *message)
+
+/**
+  Embedded library implementation of OK response.
+
+  This function is used by the server to write 'OK' packet to
+  the "network" when the server is compiled as an embedded library.
+  Since there is no network in the embedded configuration,
+  a different implementation is necessary.
+  Instead of marshalling response parameters to a network representation
+  and then writing it to the socket, here we simply copy the data to the
+  corresponding client-side connection structures. 
+
+  @sa Server implementation of net_send_ok in protocol.cc for
+  description of the arguments.
+
+  @return
+    @retval TRUE An error occurred
+    @retval FALSE Success
+*/
+
+bool
+net_send_ok(THD *thd,
+            uint server_status, uint total_warn_count,
+            ha_rows affected_rows, ulonglong id, const char *message)
 {
-  DBUG_ENTER("send_ok");
+  DBUG_ENTER("emb_net_send_ok");
   MYSQL_DATA *data;
+  bool error;
   MYSQL *mysql= thd->mysql;
-  
+
   if (!mysql)            // bootstrap file handling
-    DBUG_VOID_RETURN;
-  if (thd->net.no_send_ok)	// hack for re-parsing queries
-    DBUG_VOID_RETURN;
+    DBUG_RETURN(FALSE);
   if (!(data= thd->alloc_new_dataset()))
-    return;
+    return TRUE;
   data->embedded_info->affected_rows= affected_rows;
   data->embedded_info->insert_id= id;
   if (message)
     strmake(data->embedded_info->info, message,
             sizeof(data->embedded_info->info)-1);
 
-  write_eof_packet(thd);
-  thd->cur_data= 0;
-  DBUG_VOID_RETURN;
-}
-
-void
-send_eof(THD *thd)
-{
-  write_eof_packet(thd);
+  error= write_eof_packet(thd, server_status, total_warn_count);
   thd->cur_data= 0;
+  DBUG_RETURN(error);
 }
 
 
-void net_send_error_packet(THD *thd, uint sql_errno, const char *err)
-{
-  MYSQL_DATA *data= thd->cur_data ? thd->cur_data : thd->alloc_new_dataset();
-  struct embedded_query_result *ei= data->embedded_info;
+/**
+  Embedded library implementation of EOF response.
 
+  @sa net_send_ok
+
+  @return
+    @retval TRUE  An error occurred
+    @retval FALSE Success
+*/
+
+bool
+net_send_eof(THD *thd, uint server_status, uint total_warn_count)
+{
+  bool error= write_eof_packet(thd, server_status, total_warn_count);
+  thd->cur_data= 0;
+  return error;
+}
+
+
+bool net_send_error_packet(THD *thd, uint sql_errno, const char *err)
+{
+  MYSQL_DATA *data= thd->cur_data;
+  struct embedded_query_result *ei;
+
+  if (!thd->mysql)            // bootstrap file handling
+  {
+    fprintf(stderr, "ERROR: %d  %s\n", sql_errno, err);
+    return TRUE;
+  }
+
+  if (!data)
+    data= thd->alloc_new_dataset();
+
+  ei= data->embedded_info;
   ei->last_errno= sql_errno;
   strmake(ei->info, err, sizeof(ei->info)-1);
   strmov(ei->sqlstate, mysql_errno_to_sqlstate(sql_errno));
   ei->server_status= thd->server_status;
   thd->cur_data= 0;
+  return FALSE;
 }
 
 
-void Protocol_simple::prepare_for_resend()
+void Protocol_text::prepare_for_resend()
 {
   MYSQL_ROWS *cur;
   MYSQL_DATA *data= thd->cur_data;
@@ -1054,33 +1155,48 @@ void Protocol_simple::prepare_for_resend()
   data->embedded_info->prev_ptr= &cur->next;
   next_field=cur->data;
   next_mysql_field= data->embedded_info->fields_list;
+#ifndef DBUG_OFF
+  field_pos= 0;
+#endif
 
   DBUG_VOID_RETURN;
 }
 
-bool Protocol_simple::store_null()
+bool Protocol_text::store_null()
 {
   *(next_field++)= NULL;
   ++next_mysql_field;
   return false;
 }
 
-bool Protocol::net_store_data(const char *from, uint length)
+bool Protocol::net_store_data(const uchar *from, size_t length)
 {
   char *field_buf;
   if (!thd->mysql)            // bootstrap file handling
-    return false;
+    return FALSE;
 
-  if (!(field_buf=alloc_root(alloc, length + sizeof(uint) + 1)))
-    return true;
+  if (!(field_buf= (char*) alloc_root(alloc, length + sizeof(uint) + 1)))
+    return TRUE;
   *(uint *)field_buf= length;
   *next_field= field_buf + sizeof(uint);
-  memcpy(*next_field, from, length);
+  memcpy((uchar*) *next_field, from, length);
   (*next_field)[length]= 0;
   if (next_mysql_field->max_length < length)
     next_mysql_field->max_length=length;
   ++next_field;
   ++next_mysql_field;
-  return false;
+  return FALSE;
 }
 
+#if defined(_MSC_VER) && _MSC_VER < 1400
+#define vsnprintf _vsnprintf
+#endif
+
+int vprint_msg_to_log(enum loglevel level __attribute__((unused)),
+                       const char *format, va_list argsi)
+{
+  vsnprintf(mysql_server_last_error, sizeof(mysql_server_last_error),
+           format, argsi);
+  mysql_server_last_errno= CR_UNKNOWN_ERROR;
+  return 0;
+}
