@@ -1,4 +1,5 @@
-/* Copyright (C) 2002 MySQL AB
+/* Copyright (c) 2000-2002, 2005-2007 MySQL AB
+   Use is subject to license terms.
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -12,8 +13,9 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA */
+   Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+   MA 02110-1301, USA
+*/
 
 /*  File   : strxnmov.c
     Author : Richard A. O'Keefe.
